#define pixel_size 3

__global__ void cu_brillo(unsigned char* cu_in,unsigned char* cu_out,unsigned int step,unsigned int height,unsigned int width,float a,float b) {
	unsigned int n_pixel = blockIdx.x * blockDim.x + threadIdx.x; //me da en que numero de pixel estoy!
	if(n_pixel < height * width) {
		unsigned int n_row,n_col;
		unsigned char* pix_in,*pix_out;

		n_row = n_pixel / width;
		n_col = n_pixel % width;

		pix_in = cu_in + n_row * step + pixel_size * sizeof(unsigned char) * n_col;
		pix_out = cu_out + n_row * step + pixel_size * sizeof(unsigned char) * n_col;

		unsigned int cr,cg,cb;
		unsigned char bn,bv; //brillo nuevo y brillo viejo
		unsigned char local_pix[] = {0,0,0};

		for(int i = 0; i < 3; ++i) local_pix[i] = pix_in[i];
		
		bv = (local_pix[0] + local_pix[1] + local_pix[2])/3; 
		bn = a * bv + b;

		if(bv){
			cr = (bn * local_pix[0]) / bv;
			cg = (bn * local_pix[1]) / bv;
			cb = (bn * local_pix[2]) / bv;
		}else{
			cr = cg = cb = 0;
		}

		local_pix[0] = (cr>255)?255:cr;
		local_pix[1] = (cg>255)?255:cg;
		local_pix[2] = (cb>255)?255:cb;


		pix_out[0] = local_pix[0];
		pix_out[1] = local_pix[1]; 
		pix_out[2] = local_pix[2];
	}
}

unsigned char *cu_mem_in = 0, *cu_mem_out = 0;
bool cu_mem_init = false;

extern "C" void brillo_cuda(unsigned char* in_data,unsigned char* out_data,unsigned int step,unsigned int height,unsigned int width,float a, float b) {
	if(!cu_mem_init){
		cudaMalloc((void**)&cu_mem_in,sizeof(unsigned char) * step * height);
		cudaMalloc((void**)&cu_mem_out,sizeof(unsigned char) * step * height);
		cu_mem_init = true;
	}

	cudaMemcpy(cu_mem_in,in_data,sizeof(unsigned char) * step * height,cudaMemcpyHostToDevice);
	dim3 dimBlock(16*16,1);
	dim3 dimGrid((width * height)/dimBlock.x + 1,1); //proceso de a tiras.

	cu_brillo<<<dimGrid , dimBlock>>>(cu_mem_in,cu_mem_out,step,height,width,a,b);

	cudaMemcpy(out_data,cu_mem_out,sizeof(unsigned char) * step * height,cudaMemcpyDeviceToHost);
}

extern "C" void cuda_clean() {
	if(cu_mem_init) {
		cudaFree(cu_mem_in);
		cudaFree(cu_mem_out);
	}
}
