#include<iostream>
#include"utils.h"

using namespace std;

#define dameBrillo(pix) (((pix).r + (pix).g + (pix).b)/3)

extern "C" void brillohistoAsm(IplImage* in,IplImage* out);

void brillohisto(IplImage* in,IplImage* out){
	int* histograma = (int*)calloc(256, sizeof(int));
	unsigned int total = in->height * in->width;
	
	//inicializo el histograma
	//for(int i = 0;i<256;++i)histograma[i]=0;

	for(int irow = 0; irow < in->height;++irow){
		pixel* inrow = (pixel*)(in->imageData + in->widthStep * irow);
		for(int icol = 0;icol < in->width;++icol){
			unsigned char brillo = dameBrillo(inrow[icol]);
			++histograma[brillo];
		}
	}

	//calculo el mapeo de brillos
	unsigned int totalParcial = 0;
	for(int i = 0;i<256;++i){
		totalParcial += histograma[i]*255;	
		histograma[i] = totalParcial/total; 
	}

	//ahora reemplazo
	for(int irow = 0; irow < in->height;++irow){
		pixel* inrow = (pixel*)(in->imageData + in->widthStep * irow);
		pixel* outrow = (pixel*)(out->imageData + out->widthStep * irow);
		for(int icol = 0;icol < in->width;++icol){
			unsigned int r,g,b;
			unsigned char bn,bv; //brillo nuevo y brillo viejo

			bv = dameBrillo(inrow[icol]);
			bn = (unsigned char)histograma[bv];
			
			if(bv){
				r = (bn * inrow[icol].r) / bv;
				g = (bn * inrow[icol].g) / bv;
				b = (bn * inrow[icol].b) / bv;
			}else{
				r = g = b = 0;
			}	

			outrow[icol].r = (r>255)?255:r;
			outrow[icol].g = (g>255)?255:g;
			outrow[icol].b = (b>255)?255:b;
		}
	}
	free(histograma);
}

#ifdef _Video
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./laplaceVideo [video]" << endl; 
		return 0;
	}
	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while(1){
		frame = cvQueryFrame( capture );
            
	        if( !frame ){
		cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
	        	frame = cvQueryFrame( capture );
	        }
	        if( !frame ) break;

		if(!isinit){
#ifdef _Asm	        
			frameAlpha = agregarCanalAlpha(frame);
			salida = chequearImagen(frameAlpha);
#else
			salida = chequearImagen(frame);
#endif
			isinit = true;
		}

#ifdef _Asm	        
	        reusarCanalAlfa(frame,frameAlpha);
	        brillohistoAsm(frameAlpha, salida);
#else
	        brillohisto(frame, salida);
#endif
		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();
	return 0;
}
#else
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./brillohisto [imagein]" << endl; 
		cout << "./brillohisto [imagein] [imageout]" << endl; 
		return 0;
	}

	const char* imagename = argv[1]; 
	const char* outname = (argc<3)?0:argv[2];
	
	IplImage* img = cvLoadImage(imagename);
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");

#ifdef _Asm
	img = agregarCanalAlpha(img);
	res = chequearImagen(img);
	brillohistoAsm(img, res);
#else	
	res = chequearImagen(img);
	brillohisto(img,res);
#endif

	if(!outname){
		mostrarVentanas(img,res);
		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}
	limpiarImagenes(img,res);
	return 0;
}
#endif