#include<iostream>
#include"utils.h"

#ifdef _Cpp
void removerCanales(IplImage* in, IplImage* out, unsigned char flags [])
{	
	//En la posicion i-esima viene un 0 si el color queda y un 1 si se va

	for(int irow = 0; irow < in->height;irow++)
	{
		pixel* row = (pixel*)(in->imageData + in->widthStep * irow); //Fila actual
		pixel* outrow = (pixel*)(out->imageData + out->widthStep*irow); //Fila de salida

		for(int icol = 0;icol < in->width;icol++)
		{
			//Muestro el pixel
			pixel actual = row[icol];
			outrow[icol].r = (flags[0]) ? actual.r : 0;
			outrow[icol].g = (flags[1]) ? actual.g : 0;
			outrow[icol].b = (flags[2]) ? actual.b : 0;
		}
	}
}
#endif

#ifdef _Cuda
extern"C"{
	void removerCanales(unsigned char*,unsigned char*,unsigned int,unsigned int,unsigned int,unsigned char flags []);
	void cuda_clean();
}
#endif

#ifdef _Asm
extern"C" {
	void removerCanales(IplImage* in, IplImage* out, unsigned char flags []);
}
#endif

#ifdef _Video
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./eliminarcanales [video] bgr" << endl; 
		return 0;
	}
	if(argc<3)return 0;

	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	unsigned char color[3] = {0,0,0};
	assert(strlen(argv[2])==3);

	assert(argv[2][0]<'1'+1);
	assert(argv[2][0]>'0'-1);

	assert(argv[2][1]<'1'+1);
	assert(argv[2][1]>'0'-1);

	assert(argv[2][2]<'1'+1);
	assert(argv[2][2]>'0'-1);

	color[0] = argv[2][0]-'0';
	color[1] = argv[2][1]-'0';
	color[2] = argv[2][2]-'0';

	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while( 1 ) {
		frame = cvQueryFrame( capture );
            
	        if( !frame ){
			cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
	        	frame = cvQueryFrame( capture );
	        }
	        if( !frame ) break;

		if(!isinit){
#ifdef _Asm	        
			frameAlpha = agregarCanalAlpha(frame);
			salida = chequearImagen(frameAlpha);
#else
			salida = chequearImagen(frame);
#endif
			isinit = true;
		}

#ifdef _Asm	        
		reusarCanalAlfa(frame,frameAlpha);
	        removerCanales(frameAlpha, salida,color);
#endif
#ifdef _Cpp
	        removerCanales(frame, salida,color);
#endif
#ifdef _Cuda
	removerCanales((unsigned char*)frame->imageData,(unsigned char*)salida->imageData,frame->width,frame->height,frame->widthStep,color);
#endif
		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();
#ifdef _Cuda
	cuda_clean();
#endif
	return 0;
}

#else
int main(int argc,char** argv){
	if(argc<3)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./eliminarcanales [imagein] bgr" << endl; 
		cout << "./eliminarcanales [imagein] bgr [imageout]" << endl; 
		return 0;
	}

	unsigned char color[3] = {0,0,0};
	const char* imagename = argv[1]; 
	const char* outname = (argc<4)?0:argv[3];
	assert(strlen(argv[2])==3);

	assert(argv[2][0]<'1'+1);
	assert(argv[2][0]>'0'-1);

	assert(argv[2][1]<'1'+1);
	assert(argv[2][1]>'0'-1);

	assert(argv[2][2]<'1'+1);
	assert(argv[2][2]>'0'-1);

	color[0] = argv[2][0]-'0';
	color[1] = argv[2][1]-'0';
	color[2] = argv[2][2]-'0';

	IplImage* img = cvLoadImage(imagename);
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");

#ifdef _Asm
	IplImage* conalfa = agregarCanalAlpha(img); 
	cvReleaseImage(&img);
	img = conalfa;
	ERROR(img,"No se pudo agregar el canal de alfa");
#endif
	res = chequearImagen(img);
#ifdef _Cuda
	removerCanales((unsigned char*)img->imageData,(unsigned char*)res->imageData,img->width,img->height,img->widthStep,color);
#else
	removerCanales(img,res,color);
#endif

	if(!outname){
		mostrarVentanas(img,res);
		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}
	limpiarImagenes(img,res);
#ifdef _Cuda
	cuda_clean();
#endif
	return 0;
}
#endif
