#include<iostream>
#include"utils.h"

using namespace std;

#ifdef _Cpp
void greyscale(IplImage* in,IplImage* out){
	unsigned char* dataOut = (unsigned char*)out->imageData;	
	pixel* dataIn = (pixel*)in->imageData;	

	//width tiene la cantidadde pixels de ancho
	//widthStep es cuantos bytes hay q sumarle a un puntero para bajar de columna... mantiene la alineacion!
	
	//Esta parte es la que se vendria a optimizar en asm y CUDA, lo anterior es manejo de errores e inicializacion de estructuras.
	for(int irow = 0;irow < in->height;irow++){
		pixel* p_inrow = (pixel*)((int)dataIn + in->widthStep * irow); //calculo cual es la proxima columna
		unsigned char* p_outrow = (unsigned char*)((int)dataOut + out->widthStep * irow); 

		for(int icol = 0;icol < in->width;icol++){
			unsigned int b = (float)p_inrow[icol].b * 0.11 + (float)p_inrow[icol].g * 0.59 + (float)p_inrow[icol].r * 0.3;
			p_outrow[icol] = b; 
		}
	}
}
#elif _Cuda
extern "C"{
	void greyscale(unsigned char*,unsigned int,unsigned char*,unsigned int,unsigned int,unsigned int);
	void cuda_clean();
}
#else
extern "C"{
	void greyscale(IplImage*,IplImage*);
}
#endif

#ifdef _Video
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./greyscaleVideo [video]" << endl; 
		return 0;
	}
	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while( 1 ) {
		frame = cvQueryFrame( capture );
            
	        if( !frame ){
			cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
	        	frame = cvQueryFrame( capture );
	        }
	        if( !frame ) break;

		if(!isinit){
#ifdef _Asm	        
			frameAlpha = agregarCanalAlpha(frame);
#endif
			salida = chequearImagen_greyscale(frame);
			isinit = true;
		}

#ifdef _Asm	        
		reusarCanalAlfa(frame,frameAlpha);
	        greyscale(frameAlpha, salida);
#endif
#ifdef _Cuda
		greyscale((unsigned char*)frame->imageData,frame->widthStep,(unsigned char*)salida->imageData,salida->widthStep,frame->height,frame->width);
#endif
#ifdef _Cpp
	        greyscale(frame, salida);
#endif
		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();

#ifdef _Cuda
	cuda_clean();
#endif

	return 0;
}
#else
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./greyscale [imagein]" << endl; 
		cout << "./greyscale [imagein] [imageout]" << endl; 
		return 0;
	}

	const char* imagename = argv[1]; 
	const char* outname = (argc<3)?0:argv[2];
	
	IplImage* img = cvLoadImage(imagename);
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");
	res = chequearImagen_greyscale(img);

#ifdef _Asm
	IplImage* conalfa = agregarCanalAlpha(img);
	cvReleaseImage(&img);
	img = conalfa;
	ERROR(img,"No se pudo agregar el canal de alfa");
	greyscale(img,res);
#elif _Cuda
	greyscale((unsigned char*)img->imageData,img->widthStep,(unsigned char*)res->imageData,res->widthStep,img->height,img->width);
#else
	greyscale(img,res);
#endif

	if(!outname){
		mostrarVentanas(img,res);
		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}
	cvReleaseImage(&img);
	cvReleaseImage(&res);
#ifdef _Cuda
	cuda_clean();
#endif
	return 0;
}
#endif
