#include<iostream>
#include"utils.h"

#ifdef _Cuda
extern "C" {
	void intercambiarCanales(unsigned char*,unsigned char*,unsigned int,unsigned int,unsigned int,unsigned char[]);
	void cuda_clean();
}
#else
void intercambiarCanales(IplImage* in, IplImage* out, unsigned char orden [])
{	
	//En la posicion i-esima viene el color que tiene que quedar
	//Ej: {1,0,2}  {r,g,b} ~> {g,r,b}
	//Ej: {0,0,0}  {r,g,b} ~> {r,r,r}

	for(int irow = 0; irow < in->height;irow++)
	{
		pixel* row = (pixel*)(in->imageData + in->widthStep * irow); //Fila actual
		pixel* outrow = (pixel*)(out->imageData + out->widthStep*irow); //Fila de salida

		for(int icol = 0;icol < in->width;icol++)
		{
			//Muestro el pixel
			outrow[icol].r = *(unsigned char*)((int)&(row[icol]) + orden[2]);
			outrow[icol].g = *(unsigned char*)((int)&(row[icol]) + orden[1]);
			outrow[icol].b = *(unsigned char*)((int)&(row[icol]) + orden[0]);
		}
	}
}
#endif

extern "C" void intercambiarAsm(IplImage* in, IplImage* out, unsigned char orden []);


#ifdef _Video
int main(int argc,char** argv){
	if(argc<3)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "Uso: [video] bgr" << endl; 
		return 0;
	}
	unsigned char color[3] = {0,0,0};
	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	assert(strlen(argv[2])==3);

	assert(argv[2][0]<'2'+1);
	assert(argv[2][0]>'0'-1);

	assert(argv[2][1]<'2'+1);
	assert(argv[2][1]>'0'-1);

	assert(argv[2][2]<'2'+1);
	assert(argv[2][2]>'0'-1);

	color[0] = argv[2][0]-'0';
	color[1] = argv[2][1]-'0';
	color[2] = argv[2][2]-'0';
	
	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while(1){
		frame = cvQueryFrame( capture );
            
	        if( !frame ){
			cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
	        	frame = cvQueryFrame( capture );
	        }
	        if( !frame ) break;

		if(!isinit){
#ifdef _Asm	        
			frameAlpha = agregarCanalAlpha(frame);
			salida = chequearImagen(frameAlpha);
#else
			salida = chequearImagen(frame);
#endif
			isinit = true;
		}

#ifdef _Asm	        
		reusarCanalAlfa(frame,frameAlpha);

		intercambiarAsm(frameAlpha, salida, color);
#endif

#ifdef _Cpp
		intercambiarCanales(frame, salida, color);	
#endif
#ifdef _Cuda
		intercambiarCanales((unsigned char*)frame->imageData,(unsigned char*)salida->imageData,frame->width,frame->height,frame->widthStep,color); 
#endif

		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();
#ifdef _Cuda
	cuda_clean();
#endif
	return 0;
}
#else
int main(int argc,char** argv){
	if(argc<3)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./intercambiar [imagein] bgr" << endl; 
		cout << "./intercambiar [imagein] bgr [imageout]" << endl; 
		return 0;
	}

	unsigned char color[3] = {0,0,0};
	const char* imagename = argv[1]; 
	const char* outname = (argc<4)?0:argv[3];
	assert(strlen(argv[2])==3);

	assert(argv[2][0]<'2'+1);
	assert(argv[2][0]>'0'-1);

	assert(argv[2][1]<'2'+1);
	assert(argv[2][1]>'0'-1);

	assert(argv[2][2]<'2'+1);
	assert(argv[2][2]>'0'-1);

	color[0] = argv[2][0]-'0';
	color[1] = argv[2][1]-'0';
	color[2] = argv[2][2]-'0';

	IplImage* img = cvLoadImage(imagename);
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");

#ifdef _Asm
	img = agregarCanalAlpha(img);
	res = chequearImagen(img);
	intercambiarAsm(img,res,color);
#elif _Cuda
	res = chequearImagen(img);
	intercambiarCanales((unsigned char*)img->imageData,(unsigned char*)res->imageData,img->width,img->height,img->widthStep,color); 
#else
	res = chequearImagen(img);
	intercambiarCanales(img,res,color);
#endif

	if(!outname){
		mostrarVentanas(img,res);
		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}
	limpiarImagenes(img,res);
#ifdef _Cuda
	cuda_clean();
#endif
	return 0;
}
#endif
