#include<iostream>
#include"utils.h"

using namespace std;

extern "C" void zoomInterpoladorASM(IplImage* in, IplImage* out);
extern "C" void zoomInterpoladorCUDA(unsigned char* imgIn, unsigned char* imgOut, unsigned int widthStep, unsigned int height, unsigned int widthStepOut );

IplImage* zoomInterpolador(IplImage* in, IplImage* out)
{	
	for(int irow = 0; irow < in->height;irow++)
	{
		pixel* row = (pixel*)(in->imageData + in->widthStep * irow); //Fila actual
		pixel* outrow = (pixel*)(out->imageData + out->widthStep*irow*2); //Fila de salida

		for(int icol=0; icol < in->width; icol++)
		{
			//Copio la imagen original
			outrow[icol*2].r = row[icol].r;
			outrow[icol*2].g = row[icol].g;
			outrow[icol*2].b = row[icol].b;
		}
	}

	for(int irow = 0; irow < in->height;irow++)
	{
		pixel* row = (pixel*)(in->imageData + in->widthStep * irow); //Fila actual
		pixel* nrow = (pixel*)((int)row + in->widthStep); //Fila siguiente
		pixel* outrow = (pixel*)(out->imageData + out->widthStep*(irow*2 + 1)); //Fila de salida
		
		for(int icol=0; icol < in->width; icol++)
		{
			if (icol < in->width-1)
			{
				if (irow < in->height-1)
				{
					outrow[(icol*2)+1].r = (row[icol+1].r + row[icol].r + nrow[icol+1].r + nrow[icol].r)/4;
					outrow[(icol*2)+1].g = (row[icol+1].g + row[icol].g + nrow[icol+1].g + nrow[icol].g)/4;
					outrow[(icol*2)+1].b = (row[icol+1].b + row[icol].b + nrow[icol+1].b + nrow[icol].b)/4;
				}
				else
				{
					outrow[(icol*2)+1].r = (row[icol].r + row[icol+1].r)/2;
					outrow[(icol*2)+1].b = (row[icol].b + row[icol+1].b)/2;
					outrow[(icol*2)+1].g = (row[icol].g + row[icol+1].g)/2;					
				}
			}
			else
			{
				if (irow < in->height-1)
				{
					outrow[(icol*2)+1].r = (row[icol].r + nrow[icol].r)/2;
					outrow[(icol*2)+1].b = (row[icol].b + nrow[icol].b)/2;
					outrow[(icol*2)+1].g = (row[icol].g + nrow[icol].g)/2;
				}
				else
				{
					outrow[(icol*2)+1].r = row[icol].r;
					outrow[(icol*2)+1].b = row[icol].b;
					outrow[(icol*2)+1].g = row[icol].g;	
				}
			}
		}
	}

	for(int irow = 0; irow < out->height;irow++)
	{
		pixel* row = (pixel*)(out->imageData + out->widthStep * irow); //Fila actual

		pixel* prow;
		if (irow > 0) prow = (pixel*)((int)row - out->widthStep); //Fila anterior

		pixel* nrow;		
		if (irow < out->height-1) nrow = (pixel*)((int)row + out->widthStep); //Fila siguiente

		pixel* outrow = (pixel*)(out->imageData + out->widthStep*irow); //Fila de salida

		for(int icol = 0;icol < out->width;icol++)
		{
			if (irow % 2 == icol % 2)
				continue;

			//Estoy en el borde izquierdo
			if (icol == 0 && irow < out->height-1)
			{
				outrow[icol].r = (prow[icol].r + nrow[icol].r + row[icol+1].r)/3;
				outrow[icol].g = (prow[icol].g + nrow[icol].g + row[icol+1].g)/3;
				outrow[icol].b = (prow[icol].b + nrow[icol].b + row[icol+1].b)/3;
			}
			else if (icol < out->width-1)
			{
				//Estoy en el borde superior
				if (irow == 0)
				{
					outrow[icol].r = (row[icol-1].r + row[icol+1].r + nrow[icol].r)/3;
					outrow[icol].g = (row[icol-1].g + row[icol+1].g + nrow[icol].g)/3;
					outrow[icol].b = (row[icol-1].b + row[icol+1].b + nrow[icol].b)/3;
				}
				//Estoy en el medio de la imagen
				else if (irow < out->height-1)
				{
					outrow[icol].r = (prow[icol].r + nrow[icol].r + row[icol-1].r + row[icol+1].r)/4;
					outrow[icol].g = (prow[icol].g + nrow[icol].g + row[icol-1].g + row[icol+1].g)/4;
					outrow[icol].b = (prow[icol].b + nrow[icol].b + row[icol-1].b + row[icol+1].b)/4;
				}
				//Estoy en el borde inferior
				else
				{
					outrow[icol].r = (row[icol-1].r + row[icol+1].r + prow[icol].r)/3;
					outrow[icol].g = (row[icol-1].g + row[icol+1].g + prow[icol].g)/3;
					outrow[icol].b = (row[icol-1].b + row[icol+1].b + prow[icol].b)/3;
				}
			}
			else
			{
				//Estoy en la esquina superior derecha
				if (irow == 0)
				{
					outrow[icol].r = (row[icol-1].r + nrow[icol].r)/2;
					outrow[icol].g = (row[icol-1].g + nrow[icol].g)/2;
					outrow[icol].b = (row[icol-1].b + nrow[icol].b)/2;
				}
				//Estoy en el borde derecho
				else if (irow < out->height-1)
				{
					outrow[icol].r = (prow[icol].r + nrow[icol].r + row[icol-1].r)/3;
					outrow[icol].g = (prow[icol].g + nrow[icol].g + row[icol-1].g)/3;
					outrow[icol].b = (prow[icol].b + nrow[icol].b + row[icol-1].b)/3;
				}
			}
		}
	}
	return out;
}
#ifdef _Video
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./interpolacionVideo [video]" << endl; 
		return 0;
	}
	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while(1)
	{
		frame = cvQueryFrame(capture);
            
        if(!frame)
		{
			cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
        	frame = cvQueryFrame(capture);
        }
        if( !frame ) break;

		if(!isinit)
		{
#ifdef _Cpp	        
			salida = cvCreateImage(cvSize(frame->width*2, frame->height*2 ),IPL_DEPTH_8U,3);
#else
			frameAlpha = agregarCanalAlpha(frame);
			salida = cvCreateImage(cvSize(frame->width*2, frame->height*2 ),IPL_DEPTH_8U,4);
#endif
			if(salida){	
				salida->origin = IPL_ORIGIN_TL;
				salida->dataOrder = IPL_DATA_ORDER_PIXEL;
			}
			ERROR(salida && checkFormat(salida),"No se pudo crear correctamente el buffer para la imagen de salida");

			isinit = true;
		}

#ifdef _Asm	        
		reusarCanalAlfa(frame,frameAlpha);
		zoomInterpoladorASM(frameAlpha, salida);
#endif
#ifdef _Cpp
		salida = zoomInterpolador(frame, salida);
#endif
#ifdef _Cuda
		reusarCanalAlfa(frame,frameAlpha);
		zoomInterpoladorCUDA((unsigned char*)frameAlpha->imageData, (unsigned char*)salida->imageData, frameAlpha->widthStep, frameAlpha->height, salida->widthStep);
#endif
		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();
	return 0;
}
#else
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./interpolacion [imagein]" << endl; 
		cout << "./interpolacion [imagein] [imageout]" << endl; 
		return 0;
	}

	const char* imagename = argv[1]; 
	const char* outname = (argc<3)?0:argv[2];
	
	IplImage* img = cvLoadImage(imagename);
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");
	

	/** Creo la imagen de salida **/
	#ifdef _Cpp
	IplImage* out = cvCreateImage(cvSize(img->width*2, img->height*2 ),IPL_DEPTH_8U,3);
	#else 
	img = agregarCanalAlpha(img);
	IplImage* out = cvCreateImage(cvSize(img->width*2, img->height*2 ),IPL_DEPTH_8U,4);
	#endif
	if(out){	
		out->origin = IPL_ORIGIN_TL;
		out->dataOrder = IPL_DATA_ORDER_PIXEL;
	}
	ERROR(out && checkFormat(out),"No se pudo crear correctamente el buffer para la imagen de salida");
	/** Creo la imagen de salida **/

#ifdef _Cpp
	res = zoomInterpolador(img, out);
#endif
#ifdef _Asm
	//res = agregarCanalAlpha(out);
	res = out;
	zoomInterpoladorASM(img, res);
#endif
#ifdef _Cuda
	//res = agregarCanalAlpha(out);
	res = out;
	zoomInterpoladorCUDA((unsigned char*)img->imageData, (unsigned char*)res->imageData, img->widthStep, img->height, res->widthStep);
#endif	

	if(!outname){
		mostrarVentanas(img,res);
		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}
	limpiarImagenes(img,res);
	return 0;
}
#endif
