#include<iostream>
#include"utils.h"

using namespace std;

#define techo(expr) min(max((expr),0),255)

#ifdef _Cpp
void laplace(IplImage* in,IplImage* out){
	for(int irow = 1; irow < in->height-1;++irow){
		pixel* row = (pixel*)(in->imageData + in->widthStep * irow);
		pixel* prow = (pixel*)((int)row - in->widthStep); 
		pixel* nrow = (pixel*)((int)row + in->widthStep); 

		pixel* outrow = (pixel*)(out->imageData + out->widthStep*irow);

		for(int icol = 1;icol < in->width-1;++icol){
			outrow[icol].r = techo(row[icol].r * 8 - row[icol-1].r - row[icol+1].r - prow[icol+1].r - prow[icol-1].r - prow[icol].r - nrow[icol+1].r - nrow[icol-1].r - nrow[icol].r);
			outrow[icol].g = techo(row[icol].g * 8 - row[icol-1].g - row[icol+1].g - prow[icol+1].g - prow[icol-1].g - prow[icol].g - nrow[icol+1].g - nrow[icol-1].g - nrow[icol].g);
			outrow[icol].b = techo(row[icol].b * 8 - row[icol-1].b - row[icol+1].b - prow[icol+1].b - prow[icol-1].b - prow[icol].b - nrow[icol+1].b - nrow[icol-1].b - nrow[icol].b); 
		}
	}
}

void laplacesum(IplImage* image,IplImage* laplace,IplImage* out){
	for(int irow = 0; irow < image->height;++irow){
		pixel* row = (pixel*)(image->imageData + image->widthStep * irow);
		pixel* lrow = (pixel*)(laplace->imageData + laplace->widthStep * irow);
		pixel* outrow = (pixel*)(out->imageData + out->widthStep * irow);

		for(int icol = 0;icol < image->width;++icol){
			outrow[icol].r = techo(lrow[icol].r + row[icol].r);
			outrow[icol].g = techo(lrow[icol].g + row[icol].g);
			outrow[icol].b = techo(lrow[icol].b + row[icol].b);
		}
	}

}
#endif
#ifdef _Cuda
extern "C"{
	void laplace(unsigned char*,unsigned char*,unsigned int,unsigned int,unsigned int);
	void laplacesum(unsigned char*,unsigned char*,unsigned char*,unsigned int,unsigned int,unsigned int);
	void cuda_clean();
}
#endif
#ifdef _Asm
extern "C"{
	void laplace(IplImage* in,IplImage* out);
	void laplacesum(IplImage* image,IplImage* laplace,IplImage* out);
}
#endif

#ifdef _Video
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./laplaceVideo [video]" << endl; 
		return 0;
	}
	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while(1){
		frame = cvQueryFrame( capture );
            
	        if( !frame ){
			cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
	        	frame = cvQueryFrame( capture );
	        }
	        if( !frame ) break;

		if(!isinit){
#ifdef _Asm	        
			frameAlpha = agregarCanalAlpha(frame);
			salida = chequearImagen(frameAlpha);
#else
			salida = chequearImagen(frame);
#endif
			isinit = true;
		}

#ifdef _Asm	        
		reusarCanalAlfa(frame,frameAlpha);
	        laplace(frameAlpha, salida);
#endif
#ifdef _Cpp
	        laplace(frame, salida);
#endif
#ifdef _Cuda
	laplace((unsigned char*)frame->imageData,(unsigned char*)salida->imageData,frame->widthStep,frame->height,frame->width);	
#endif
		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();
#ifdef _Cuda
	cuda_clean();
#endif
	return 0;
}
#else
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./laplace [imagein]" << endl; 
		cout << "./laplace [imagein] [imageout]" << endl; 
		return 0;
	}

	const char* imagename = argv[1]; 
	const char* outname = (argc<3)?0:argv[2];

	IplImage* img = cvLoadImage(imagename);
	IplImage* lap = 0;
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");
#ifdef _Asm
	IplImage* conalfa = agregarCanalAlpha(img); 
	cvReleaseImage(&img);
	img = conalfa;
	ERROR(img,"No se pudo agregar el canal de alfa");
#endif
	
	lap = chequearImagen(img);
	res = chequearImagen(img);

#ifdef _Cuda
	laplace((unsigned char*)img->imageData,(unsigned char*)lap->imageData,img->widthStep,img->height,img->width);
	laplacesum((unsigned char*)img->imageData,(unsigned char*)lap->imageData,(unsigned char*)res->imageData,img->widthStep,img->height,img->width);
#else
	laplace(img,lap);
	laplacesum(img,lap,res);
#endif

	if(!outname){
		cvNamedWindow("Original",CV_WINDOW_AUTOSIZE);	
		cvShowImage("Original",img);	

		cvNamedWindow("Bordes",CV_WINDOW_AUTOSIZE);	
		cvShowImage("Bordes",lap);	

		cvNamedWindow("Procesada",CV_WINDOW_AUTOSIZE);	
		cvShowImage("Procesada",res);	


		cvMoveWindow("Original", 20, 30);
		cvMoveWindow("Bordes", 40+(img->width), 30);
		cvMoveWindow("Procesada", 20, 70 + (img->height));
		
		cvWaitKey(0);

		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}

	limpiarImagenes(img,res);
#ifdef _Cuda
	cuda_clean();
#endif
	return 0;
}
#endif
