#include<iostream>
#include"utils.h"

using namespace std;

extern "C" void pixelizarASM(IplImage* in, IplImage* out);
extern "C" void pixelizarCUDA(unsigned int* imgIn, unsigned int* imgOut, unsigned int widthStep, unsigned int height );

void pixelizar(IplImage* in,IplImage* out){
	pixel* dataOut = (pixel*)out->imageData;	
	pixel* dataIn = (pixel*)in->imageData;	
	
	pixel ultimoPixel; //Pixel temporal para guardar el que voy a repetir.
	pixel* p_inrow; //Puntero a la fila que estoy leyendo.
	pixel* p_outrow; //Puntero a la fila que estoy escribiendo.
	int filas = in->height; //Cantidad de filas.
	int cols = in->width; //Cantidad de columnas.
	int widthStep = in->widthStep; //Cantidad de bytes por fila.
	int TAM_PIX = 8; //Tamaño del pixel resultante.
	
	//Cada TAM_PIX filas:
	for(int irow = 0;irow < filas;irow+=TAM_PIX){	
		//Calculo la fila y la columna que voy a leer.
		p_inrow = (pixel*)((int)dataIn + in->widthStep * irow);
		p_outrow = (pixel*)((int)dataOut + in->widthStep * irow);
		
		//Por cada columna:
		for(int icol = 0;icol < cols;icol++)
		{
			//Si estoy en un pixel que tiene que ser repetido lo guardo.
			if (icol % TAM_PIX == 0)
				ultimoPixel = p_inrow[icol];
			
			//Copio el pixel en las TAM_PIX filas hacia abajo.
			for(int i = 0; i < TAM_PIX; i++)
			{
				if (irow + i < filas)
					((pixel*)((int)dataOut + in->widthStep * (irow + i)))[icol] = ultimoPixel;
			}
		}
	}
}

#ifdef _Video
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./gauss [video]" << endl; 
		return 0;
	}
	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while(1){
		frame = cvQueryFrame( capture );
            
	        if( !frame ){
			cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
	        	frame = cvQueryFrame( capture );
	        }
	        if( !frame ) break;

		if(!isinit){
#ifdef _Cpp	        
			salida = chequearImagen(frame);
#else
			frameAlpha = agregarCanalAlpha(frame);
			salida = chequearImagen(frameAlpha);			
#endif
			isinit = true;
		}

#ifdef _Asm	        
		reusarCanalAlfa(frame,frameAlpha);

		pixelizarASM(frameAlpha, salida);
#endif

#ifdef _Cuda
		reusarCanalAlfa(frame,frameAlpha);
		pixelizarCUDA((unsigned int*)frameAlpha->imageData,(unsigned int*)salida->imageData, frameAlpha->widthStep, frameAlpha->height);

#endif

#ifdef _Cpp
		pixelizar(frame, salida);	
#endif
		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();
	return 0;
}
#else
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./pixelizar [imagein]" << endl; 
		cout << "./pixelizar [imagein] [imageout]" << endl; 
		return 0;
	}
	const char* imagename = argv[1]; 
	const char* outname = (argc<3)?0:argv[2];
	
	IplImage* img = cvLoadImage(imagename);
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");
#ifdef _Cpp
	res = chequearImagen(img);
	pixelizar(img,res);
#endif
#ifdef _Asm
	IplImage* alfa;
	alfa = agregarCanalAlpha(img);
	cvReleaseImage(&img);
	img = alfa;
	res = chequearImagen(img);
	pixelizarASM(img,res);
#endif
#ifdef _Cuda
	IplImage* alfa;
	alfa = agregarCanalAlpha(img);
	cvReleaseImage(&img);
	img = alfa;
	res = chequearImagen(img);
	pixelizarCUDA((unsigned int*)img->imageData,(unsigned int*)res->imageData, img->widthStep, img->height);
#endif
	if(!outname){
		mostrarVentanas(img,res);
		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}
	limpiarImagenes(img,res);
	return 0;
}
#endif
