#include<iostream>
#include"utils.h"

using namespace std;

#define techo(expr) (min(255.0,max(0.0,(expr))))


#ifdef _Cpp
void rgbtohsl(IplImage* in,IplImage* out,int sat){
	IplImage* temp = NULL;
	temp = chequearImagen(in);
		
	
	pixel* dataTemp = (pixel*)temp->imageData;	
	pixel* dataOut = (pixel*)out->imageData;	
	pixel* dataIn = (pixel*)in->imageData;	
	
	pixel* p_inrow; //Puntero a la fila que estoy leyendo.
	pixel* p_outrow; //Puntero a la fila que estoy escribiendo.
	int filas = in->height; //Cantidad de filas.
	int cols = in->width; //Cantidad de columnas.
	int widthStep = in->widthStep; //Cantidad de bytes por fila.	

	
	for(int irow = 0;irow < filas;irow++){
		p_inrow = (pixel*)((int)dataIn + widthStep * irow); //calculo cual es la proxima columna
		p_outrow = (pixel*)((int)dataTemp + widthStep * irow); 

		//de rgb a hsl

		for(int icol = 0;icol < cols;icol++){
			float r, g, b, h, s, l; 
			r = p_inrow[icol].r / 256.0; 
			g = p_inrow[icol].g / 256.0; 
			b = p_inrow[icol].b / 256.0;
			
			float maxColor = max(r, max(g, b)); 
			float minColor = min(r, min(g, b));
			
			
			if(minColor == maxColor){
			   	h = 0.0; 
				s = 0.0;       
				l = r; 
			}else{   
				l = (minColor + maxColor) / 2;     
        
				if(l < 0.5){
					s = (maxColor - minColor) / (maxColor + minColor);
				}else{
					s = (maxColor - minColor) / (2.0 - maxColor - minColor);
				}
        
				if(r == maxColor){
					h = (g - b) / (maxColor - minColor);
				}else if(g == maxColor){
					h = 2.0 + (b - r) / (maxColor - minColor);
				}else{
					h = 4.0 + (r - g) / (maxColor - minColor);
				}
        
				h /= 6;
				
			if(h < 0) h ++;
			}
			
			p_outrow[icol].r = int(h * 255.0);
			p_outrow[icol].b = int( techo( s * 255.0 + sat));
			p_outrow[icol].g = int(l * 255.0);
		}
	}

	// de hsl a rgb, con la saturacion modificada
	
	for(int irow = 0;irow < filas;irow++){
		p_inrow = (pixel*)((int)dataTemp + widthStep * irow); 
		p_outrow = (pixel*)((int)dataOut + widthStep * irow); 

		for(int icol = 0;icol < cols;icol++){
			
			float r, g, b, h, s, l; 
			float temp1, temp2, tempr, tempg, tempb;
			h = p_inrow[icol].r / 256.0;
			s = p_inrow[icol].b / 256.0;
			l = p_inrow[icol].g / 256.0;

			if(s == 0){
				r = g = b = l;
			}else{
				if(l < 0.5) temp2 = l * (1 + s);      
				else temp2 = (l + s) - (l * s);     
				temp1 = 2 * l - temp2;    
				tempr = h + 1.0 / 3.0;    
				if(tempr > 1) tempr--;
				tempg = h;     
				tempb = h - 1.0 / 3.0;
				if(tempb < 0) tempb++; 
        
				if(tempr < 1.0 / 6.0) r = temp1 + (temp2 - temp1) * 6.0 * tempr;      
				else if(tempr < 0.5) r = temp2;   
				else if(tempr < 2.0 / 3.0) r = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - tempr) * 6.0;
				else r = temp1; 
        
				if(tempg < 1.0 / 6.0) g = temp1 + (temp2 - temp1) * 6.0 * tempg;    
				else if(tempg < 0.5) g = temp2;
				else if(tempg < 2.0 / 3.0) g = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - tempg) * 6.0;
				else g = temp1; 
        
				if(tempb < 1.0 / 6.0) b = temp1 + (temp2 - temp1) * 6.0 * tempb;   
				else if(tempb < 0.5) b = temp2; 
				else if(tempb < 2.0 / 3.0) b = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - tempb) * 6.0;    
				else b = temp1;
			}


			p_outrow[icol].r = int(r * 255.0);
			p_outrow[icol].g = int(g * 255.0);
			p_outrow[icol].b = int(b * 255.0);
		}
	}
	cvReleaseImage(&temp);
}
#elif _Cuda
extern "C" {
	void rgbtohsl(unsigned char*,unsigned char*,unsigned int,unsigned int,unsigned int,int);
	void cuda_clean();	
}
#endif

#ifdef _Video
int main(int argc,char** argv){
	if(argc < 2 || !strcmp(argv[1],"--help") || argc < 3){
		cout << "./rgbtohsl [Video] [int sat]" << endl; 
		return 0;
	}

	int sat = atoi(argv[2]);
	
	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while( 1 ) {
		frame = cvQueryFrame( capture );
            
	        if( !frame ){
				cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
	        	frame = cvQueryFrame( capture );
	        }
	        if( !frame ) break;

		if(!isinit){
			salida = chequearImagen(frame);
			isinit = true;
		}

#ifdef _Cpp
		rgbtohsl(frame,salida,sat);
#endif
#ifdef _Cuda
		rgbtohsl((unsigned char*)frame->imageData,(unsigned char*)salida->imageData,frame->width,frame->height,frame->widthStep,sat);
#endif
		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();
#ifdef _Cuda
	cuda_clean();
#endif
	return 0;
}

#else

int main(int argc,char** argv){
	if(argc < 2 || !strcmp(argv[1],"--help") || argc < 3){
		cout << "./rgbtohsl [imagein] [int sat]" << endl; 
		cout << "./rgbtohsl [imagein] [imageout] [int sat]" << endl; 
		return 0;
	}

	const char* imagename = argv[1]; 
	const char* outname = (argc<4)?0:argv[2];
	int sat = atoi(argv[(outname)?3:2]);
	
	IplImage* img = cvLoadImage(imagename);
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");
	res = chequearImagen(img);

#ifdef _Cpp
	rgbtohsl(img,res,sat);
#elif _Cuda
	rgbtohsl((unsigned char*)img->imageData,(unsigned char*)res->imageData,img->width,img->height,img->widthStep,sat);
#endif

	if(!outname){
		mostrarVentanas(img,res);
		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}
	limpiarImagenes(img,res);
	return 0;
}
#endif
