#include<iostream>
#include"utils.h"

using namespace std;
extern "C" void sobelASM(IplImage* in, IplImage* out);
extern "C" void sobelCUDA(unsigned char* imgIn, unsigned char* imgOut, unsigned int widthStep, unsigned int height );
void sobel(IplImage* in,IplImage* out)
{
	for(int irow = 1; irow < in->height-1;++irow)
	{
		pixel* row = (pixel*)(in->imageData + in->widthStep * irow); //Fila actual
		pixel* prow = (pixel*)((int)row - in->widthStep); //Fila anterior
		pixel* nrow = (pixel*)((int)row + in->widthStep); //Fila siguiente

		pixel* outrow = (pixel*)(out->imageData + out->widthStep*irow); //Fila de salida

		for(int icol = 1;icol < in->width-1;++icol)
		{	
			int resultado;
			pixel kernel0;
			pixel kernel1;

			resultado = prow[icol-1].r + 2 * prow[icol].r + prow[icol+1].r - nrow[icol-1].r - 2 * nrow[icol].r - nrow[icol+1].r;
			if (resultado < 0) resultado = 0;
			else if (resultado >255) resultado = 255;			
			kernel0.r = (unsigned char)resultado;
			
			resultado = prow[icol-1].g + 2 * prow[icol].g + prow[icol+1].g - nrow[icol-1].g - 2 * nrow[icol].g - nrow[icol+1].g;
			if (resultado < 0) resultado = 0;
			else if (resultado >255) resultado = 255;			
			kernel0.g = (unsigned char)resultado;			

			resultado = prow[icol-1].b + 2 * prow[icol].b + prow[icol+1].b - nrow[icol-1].b - 2 * nrow[icol].b - nrow[icol+1].b;
			if (resultado < 0) resultado = 0;
			else if (resultado >255) resultado = 255;			
			kernel0.b = (unsigned char)resultado;

			resultado = (-1) * prow[icol-1].r + prow[icol+1].r - 2 * row[icol-1].r + 2 * row[icol+1].r - nrow[icol-1].r + nrow[icol+1].r;
			if (resultado < 0) resultado = 0;
			else if (resultado >255) resultado = 255;			
			kernel1.r = (unsigned char)resultado;
			
			resultado = (-1) * prow[icol-1].g + prow[icol+1].g - 2 * row[icol-1].g + 2 * row[icol+1].g - nrow[icol-1].g + nrow[icol+1].g;
			if (resultado < 0) resultado = 0;
			else if (resultado >255) resultado = 255;			
			kernel1.g = (unsigned char)resultado;

			resultado = (-1) * prow[icol-1].b + prow[icol+1].b - 2 * row[icol-1].b + 2 * row[icol+1].b - nrow[icol-1].b + nrow[icol+1].b;
			if (resultado < 0) resultado = 0;
			else if (resultado > 255) resultado = 255;			
			kernel1.b = (unsigned char)resultado;

			//Muestro el pixel
			outrow[icol].r = sqrt ( pow(kernel0.r, 2) + pow(kernel1.r, 2) );
			outrow[icol].g = sqrt ( pow(kernel0.g, 2) + pow(kernel1.g, 2) );
			outrow[icol].b = sqrt ( pow(kernel0.b, 2) + pow(kernel1.b, 2) );
		}
	}

}
#ifdef _Video
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./laplaceVideo [video]" << endl; 
		return 0;
	}
	bool isinit = false;
	IplImage  *frame;
	IplImage  *frameAlpha;
	IplImage  *salida;
	int       key;

	CvCapture *capture = cvCaptureFromAVI( argv[1] );
	if( !capture ){
		cout <<  "No se pudo abrir el archivo " << argv[1] << endl;
		return -1;
	}    

	int fps = (int)cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	cvNamedWindow( "Video Original", CV_WINDOW_AUTOSIZE );
	cvNamedWindow( "Video Salida", CV_WINDOW_AUTOSIZE );

	while(1){
		frame = cvQueryFrame( capture );
            
	        if( !frame ){
			cvSetCaptureProperty(capture, CV_CAP_PROP_POS_AVI_RATIO, 0);
	        	frame = cvQueryFrame( capture );
	        }
	        if( !frame ) break;

		if(!isinit){
#ifdef _Cpp	        
			salida = chequearImagen(frame);
#else
			frameAlpha = agregarCanalAlpha(frame);
			salida = chequearImagen(frameAlpha);			
#endif
			isinit = true;
		}

#ifdef _Asm	        
	        reusarCanalAlfa(frame,frameAlpha);
	        sobelASM(frameAlpha, salida);
#endif
#ifdef _Cpp
	        sobel(frame, salida);
#endif
#ifdef _Cuda
		reusarCanalAlfa(frame,frameAlpha);
		sobelCUDA((unsigned char*)frameAlpha->imageData,(unsigned char*)salida->imageData, frameAlpha->widthStep, frameAlpha->height);
#endif
		cvShowImage( "Video Original", frame );
		cvShowImage( "Video Salida", salida );

		key = cvWaitKey( 1000 / fps ) & 0xFF;
		if(key == 27) break;
	}
	cvReleaseCapture(&capture);
#ifdef _Asm
	cvReleaseImage(&frameAlpha);
#endif
	cvReleaseImage(&salida);
	limpiarVentanas();
	return 0;
}
#else
int main(int argc,char** argv){
	if(argc<2)return 0;
	if(!strcmp(argv[1],"--help")){
		cout << "./sobel [imagein]" << endl; 
		cout << "./sobel [imagein] [imageout]" << endl; 
		return 0;
	}

	const char* imagename = argv[1]; 
	const char* outname = (argc<3)?0:argv[2];
	
	IplImage* img = cvLoadImage(imagename);
	IplImage* res = 0;

	ERROR(img,"No se pudo abrir la imagen");
	res = chequearImagen(img);
	
#ifdef _Cpp
	sobel(img,res);
#endif
#ifdef _Asm
	res = agregarCanalAlpha(res);
	img = agregarCanalAlpha(img);
	sobelASM(img, res);
#endif
#ifdef _Cuda
	res = agregarCanalAlpha(res);
	img = agregarCanalAlpha(img);
	sobelCUDA((unsigned char*)img->imageData,(unsigned char*)res->imageData, img->widthStep, img->height);
#endif
	
	if(!outname){
		mostrarVentanas(img,res);
		limpiarVentanas();
	}else{
		cvSaveImage(outname,res);
		cout << "Imagen guardada como " << outname << endl;
	}
	limpiarImagenes(img,res);
	return 0;
}
#endif
