#include"highgui.h"
#include"cv.h"
#include<iostream>
#include <math.h>
#include <sys/time.h>

using namespace cv;
using namespace std;

struct pixel{
	unsigned char r,g,b;
};

struct pixelalfa{
	unsigned char r,g,b,a;
};

bool checkFormat(IplImage* img){
	//3 canales de 8 bits sin canal de alfa
	//El origen esta en la parte superior izquierda
	//Interleaved ( se organiza la imagen en pixels y no en planos separados para c/canal )	
	//Veo que este alineada la memoria para hacer los mov alineados
	bool b = img->depth == IPL_DEPTH_8U && img->nChannels >= 3 
	&& img->origin == IPL_ORIGIN_TL && img->dataOrder == IPL_DATA_ORDER_PIXEL &&
	((int)img->imageData & 0x00000008)==0;
	return b;
}

void ERROR(bool b,const char* str = 0,void(OnExit)(void*) = 0,void* args = 0){
	//OnExit es un puntero a una funcion, que devuelve void, y que toma un puntero a void con los parametros (castear a lo loco...)
	//el objetivo es pasarle una funcion de limpieza por si se produce un error, asi evitar memory leaks.
	if(!b){
		if(str)cout << str << endl;
		if(OnExit)OnExit(args);
		exit(-1);
	}
}

void ReleaseImages(void* imgs){ // si se produce un error en el formato, llama a esta funcion q limpia la memoria reservada para la imagen.
	IplImage** ptrs = (IplImage**)imgs;
	for(int i = 0;ptrs[i]!=0;++i){
		cvReleaseImage(&ptrs[i]);
	}	
}

void reusarCanalAlfa(IplImage* fram,IplImage* alfa){
	pixel* dataIn = (pixel*)fram->imageData;	
	pixelalfa* dataOut = (pixelalfa*)alfa->imageData;

	for(int irow = 0;irow < fram->height;irow++){
		pixel* p_inrow = (pixel*)((int)dataIn + fram->widthStep * irow); //calculo cual es la proxima columna
		pixelalfa* p_outrow = (pixelalfa*)((int)dataOut + alfa->widthStep * irow); 

		for(int icol = 0;icol < fram->width;icol++){
			p_outrow[icol].b = p_inrow[icol].b; 
			p_outrow[icol].g = p_inrow[icol].g; 
			p_outrow[icol].r = p_inrow[icol].r; 
			p_outrow[icol].a = 0; 
		}
	}

}

IplImage* agregarCanalAlpha(IplImage* image){
	IplImage* res = cvCreateImage(cvGetSize(image),IPL_DEPTH_8U, 4);
	reusarCanalAlfa(image,res);
	return res;
}


IplImage* chequearImagen(IplImage* in){
	void* imgs[] = {in,0,0}; //meto las imagenes q voy reservando en este array

	ERROR(checkFormat(in),"Mal formato de imagen",&ReleaseImages,(void*)imgs);

	IplImage* out = cvCreateImage(cvGetSize(in),IPL_DEPTH_8U, in->nChannels);
	
	if(out)
	{	
		out->origin = IPL_ORIGIN_TL;
		out->dataOrder == IPL_DATA_ORDER_PIXEL;
	}	
	imgs[1] = (void*)out;

	ERROR(out && checkFormat(out),"No se pudo crear correctamente el buffer para la imagen de salida",&ReleaseImages,(void*)imgs);
	
	return out;
}

bool checkFormat_greyscale(IplImage* img){
	//1 canal de 8 bits sin canal de alfa
	//El origen esta en la parte superior izquierda
	//Interleaved ( se organiza la imagen en pixels y no en planos separados para c/canal )	
	//Veo que este alineada la memoria para hacer los mov alineados
	bool b = img->depth == IPL_DEPTH_8U && img->nChannels == 1 
	&& img->origin == IPL_ORIGIN_TL && img->dataOrder == IPL_DATA_ORDER_PIXEL &&
	((int)img->imageData & 0x00000008)==0;
	return b;
}


IplImage* chequearImagen_greyscale(IplImage* in){
	void* imgs[] = {in,0,0}; //meto las imagenes q voy reservando en este array

	ERROR(checkFormat(in),"Mal formato de imagen",&ReleaseImages,(void*)imgs);

	IplImage* out = cvCreateImage(cvGetSize(in),IPL_DEPTH_8U, 1);
	
	if(out){	
		out->origin = IPL_ORIGIN_TL;
		out->dataOrder = IPL_DATA_ORDER_PIXEL;
	}	
	imgs[1] = (void*)out;

	ERROR(out && checkFormat_greyscale(out),"No se pudo crear correctamente el buffer para la imagen de salida",&ReleaseImages,(void*)imgs);
	return out;
}

void mostrarVentanas(IplImage* image,IplImage* res){
	//inicializo 2 ventanas, una para mostrar el input y la otra para el output
	cvNamedWindow("Original",CV_WINDOW_AUTOSIZE);	
	cvShowImage("Original",image);	

	cvNamedWindow("Procesada",CV_WINDOW_AUTOSIZE);	
	cvShowImage("Procesada",res);	

	cvMoveWindow("Original", 20, 30);
	cvMoveWindow("Procesada", 40+(image->width), 30);
	//cuando se aprieta una tecla salgo
	cvWaitKey(0);
}

void limpiarVentanas(){
	cvDestroyAllWindows();
}

void limpiarImagenes(IplImage *img,IplImage* res){
	cvReleaseImage(&img);
	cvReleaseImage(&res);
}

unsigned long long rdtsc(){
	//en edx:eax esta el valor del tsc
	asm volatile("rdtsc");
}

double medir() {
	static struct timeval last;
	struct timeval now;
	double dif;

	gettimeofday(&now,NULL);
	dif = (double)(now.tv_sec - last.tv_sec)*1000.0 + (double)(now.tv_usec - last.tv_usec) / 1000.0;
	last = now;

	return dif;
}
